import { DataSource, ViewColumn, ViewEntity } from "typeorm";
import { memberExecutivePositions } from "../entity/club/member/memberExecutivePositions";
import { DB_TYPE } from "../env.defaults";

@ViewEntity({
  expression: (datasource: DataSource) =>
    datasource
      .getRepository(memberExecutivePositions)
      .createQueryBuilder("memberExecutivePositions")
      .select("executivePosition.id", "positionId")
      .addSelect("executivePosition.position", "position")
      .addSelect("member.id", "memberId")
      .addSelect("member.firstname", "memberFirstname")
      .addSelect("member.lastname", "memberLastname")
      .addSelect("member.nameaffix", "memberNameaffix")
      .addSelect("member.birthdate", "memberBirthdate")
      .addSelect("salutation.salutation", "memberSalutation")
      .addSelect(
        DB_TYPE == "postgres"
          ? `SUM(COALESCE("memberExecutivePositions"."end", CURRENT_DATE) - "memberExecutivePositions"."start")`
          : "SUM(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURRENT_DATE)))",
        "durationInDays"
      )
      .addSelect(
        DB_TYPE == "postgres"
          ? `SUM(AGE(COALESCE("memberExecutivePositions"."end", CURRENT_DATE), "memberExecutivePositions"."start"))`
          : "CONCAT('_', FROM_DAYS(SUM(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURRENT_DATE)))))",
        "durationInYears"
      )
      .leftJoin("memberExecutivePositions.executivePosition", "executivePosition")
      .leftJoin("memberExecutivePositions.member", "member")
      .leftJoin("member.salutation", "salutation")
      .groupBy("executivePosition.id")
      .addGroupBy("member.id")
      .addGroupBy("salutation.id"),
})
export class memberExecutivePositionsView {
  @ViewColumn()
  durationInDays: number;

  @ViewColumn()
  durationInYears: string;

  @ViewColumn()
  position: string;

  @ViewColumn()
  positionId: number;

  @ViewColumn()
  memberId: string;

  @ViewColumn()
  memberSalutation: string;

  @ViewColumn()
  memberFirstname: string;

  @ViewColumn()
  memberLastname: string;

  @ViewColumn()
  memberNameaffix: string;

  @ViewColumn()
  memberBirthdate: Date;
}
